-------------------------------------------------------------------------------
--
-- Inverse Mix Columns Module 
-- Written for 128-bit AES decryption project
-- Course: CSEE 4840 - Embedded System Design, Spring 2008
-- Authors: Shrivathsa Bhargav (sb2784)
--			Larry Chen (lc2454)
--			Abhinandan Majumdar (am2993)
--			Shiva Ramudit (syr9)
--
-- Last modified: 5-8-2008
-------------------------------------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- Performs the mixcolumns module of the code
-- We specify all of the values of the for loop as inputs and outputs that would be in the C code
entity inv_mixcolumns is
	port (
		it : in std_logic_vector (127 downto 0);
		ot : out std_logic_vector (127 downto 0)
		);
end;

architecture inv_mixcolumns_arch of inv_mixcolumns is

-- Used in the Multiply calculation
signal multconst0 : std_logic_vector (7 downto 0) := x"0E"; -- 0x0E
signal multconst1 : std_logic_vector (7 downto 0) := x"0B"; -- 0x0B
signal multconst2 : std_logic_vector (7 downto 0) := x"0D"; -- 0x0D
signal multconst3 : std_logic_vector (7 downto 0) := x"09"; -- 0x09

signal mainconst :std_logic_vector (7 downto 0) := x"FF"; -- 0xFF

signal it00, it01, it02, it03 : std_logic_vector (7 downto 0);
signal it10, it11, it12, it13 : std_logic_vector (7 downto 0);
signal it20, it21, it22, it23 : std_logic_vector (7 downto 0);
signal it30, it31, it32, it33 : std_logic_vector (7 downto 0);
signal ot00, ot01, ot02, ot03 : std_logic_vector (7 downto 0);
signal ot10, ot11, ot12, ot13 : std_logic_vector (7 downto 0);
signal ot20, ot21, ot22, ot23 : std_logic_vector (7 downto 0);
signal ot30, ot31, ot32, ot33 : std_logic_vector (7 downto 0);

begin

it00 <= it(127 downto 120);
it01 <= it(119 downto 112);
it02 <= it(111 downto 104);
it03 <= it(103 downto 96);
it10 <= it(95 downto 88);
it11 <= it(87 downto 80);
it12 <= it(79 downto 72);
it13 <= it(71 downto 64);
it20 <= it(63 downto 56);
it21 <= it(55 downto 48);
it22 <= it(47 downto 40);
it23 <= it(39 downto 32);
it30 <= it(31 downto 24);
it31 <= it(23 downto 16);
it32 <= it(15 downto 8);
it33 <= it(7 downto 0);

multconst0 <= multconst0; 
multconst1 <= multconst1; 
multconst2 <= multconst2; 
multconst3 <= multconst3; 
mainconst <= mainconst;

-- Perform all 16 Row Calculations
Out00: entity work.inv_multiply_row port map(
		input0 => it00, const0 => multconst0, 
		input1 => it10, const1 => multconst1, 
		input2 => it20, const2 => multconst2, 
		input3 => it30, const3 => multconst3, 
		output => ot00);
		
Out10: entity work.inv_multiply_row port map(
		input0 => it00, const0 => multconst3, 
		input1 => it10, const1 => multconst0, 
		input2 => it20, const2 => multconst1, 
		input3 => it30, const3 => multconst2, 
		output => ot10);

Out20: entity work.inv_multiply_row port map(
		input0 => it00, const0 => multconst2, 
		input1 => it10, const1 => multconst3, 
		input2 => it20, const2 => multconst0, 
		input3 => it30, const3 => multconst1, 
		output => ot20);
		
Out30: entity work.inv_multiply_row port map(
		input0 => it00, const0 => multconst1, 
		input1 => it10, const1 => multconst2, 
		input2 => it20, const2 => multconst3, 
		input3 => it30, const3 => multconst0, 
		output => ot30);

Out01: entity work.inv_multiply_row port map(
		input0 => it01, const0 => multconst0, 
		input1 => it11, const1 => multconst1, 
		input2 => it21, const2 => multconst2, 
		input3 => it31, const3 => multconst3, 
		output => ot01);
		
Out11: entity work.inv_multiply_row port map(
		input0 => it01, const0 => multconst3, 
		input1 => it11, const1 => multconst0, 
		input2 => it21, const2 => multconst1, 
		input3 => it31, const3 => multconst2, 
		output => ot11);

Out21: entity work.inv_multiply_row port map(
		input0 => it01, const0 => multconst2, 
		input1 => it11, const1 => multconst3, 
		input2 => it21, const2 => multconst0, 
		input3 => it31, const3 => multconst1, 
		output => ot21);
		
Out31: entity work.inv_multiply_row port map(
		input0 => it01, const0 => multconst1, 
		input1 => it11, const1 => multconst2, 
		input2 => it21, const2 => multconst3, 
		input3 => it31, const3 => multconst0, 
		output => ot31);
		
Out02: entity work.inv_multiply_row port map(
		input0 => it02, const0 => multconst0, 
		input1 => it12, const1 => multconst1, 
		input2 => it22, const2 => multconst2, 
		input3 => it32, const3 => multconst3, 
		output => ot02);
		
Out12: entity work.inv_multiply_row port map(
		input0 => it02, const0 => multconst3, 
		input1 => it12, const1 => multconst0, 
		input2 => it22, const2 => multconst1, 
		input3 => it32, const3 => multconst2, 
		output => ot12);

Out22: entity work.inv_multiply_row port map(
		input0 => it02, const0 => multconst2, 
		input1 => it12, const1 => multconst3, 
		input2 => it22, const2 => multconst0, 
		input3 => it32, const3 => multconst1, 
		output => ot22);
		
Out32: entity work.inv_multiply_row port map(
		input0 => it02, const0 => multconst1, 
		input1 => it12, const1 => multconst2, 
		input2 => it22, const2 => multconst3, 
		input3 => it32, const3 => multconst0, 
		output => ot32);

Out03: entity work.inv_multiply_row port map(
		input0 => it03, const0 => multconst0, 
		input1 => it13, const1 => multconst1, 
		input2 => it23, const2 => multconst2, 
		input3 => it33, const3 => multconst3, 
		output => ot03);
		
Out13: entity work.inv_multiply_row port map(
		input0 => it03, const0 => multconst3, 
		input1 => it13, const1 => multconst0, 
		input2 => it23, const2 => multconst1, 
		input3 => it33, const3 => multconst2, 
		output => ot13);

Out23: entity work.inv_multiply_row port map(
		input0 => it03, const0 => multconst2, 
		input1 => it13, const1 => multconst3, 
		input2 => it23, const2 => multconst0, 
		input3 => it33, const3 => multconst1, 
		output => ot23);
		
Out33: entity work.inv_multiply_row port map(
		input0 => it03, const0 => multconst1, 
		input1 => it13, const1 => multconst2, 
		input2 => it23, const2 => multconst3, 
		input3 => it33, const3 => multconst0, 
		output => ot33);

ot <= ot00 & ot01 & ot02 & ot03 &
		 ot10 & ot11 & ot12 & ot13 &
		 ot20 & ot21 & ot22 & ot23 &
		 ot30 & ot31 & ot32 & ot33 ;

end inv_mixcolumns_arch; 